<div class="container" style="-webkit-transform: translate(1300px, 950px) rotate(80deg);">

<div class="point reachable" style="-webkit-transform: translate(0, -300px); width: 1500px; height: 400px"><span>●</span> Rendering cache</div>

<div class="slide reachable" id="render-cache-arrays" markdown="1">

# Renderable arrays

* Generalization of renderable arrays is one of the major clean-ups of the rendering pipeline of Drupal 7.
* Even the whole page is a renderable array, which follows a standard structure:
 <code>
$page['page_top']
$page['header']
$page['sidebar_first']
$page['content']
$page['sidebar_second']
$page['page_bottom']
</code>
* The heavy processing (execution of SQL queries, etc.) can be delayed as late as possible using <code>#pre_render</code> callbacks.

</div>

<div class="slide reachable" id="render-cache-property" markdown="1" style="left: 500px;">

# The render cache

* The <code>#cache</code> property is intercepted during the rendering by <code>drupal_render()</code>, which tries to pull part of the render tree from the cache before proceeding to render on miss.
* The cache behavior can be controlled in a fine-grained manner by the keys of <code>#cache</code>:
 * **cid** and **keys / granularity**
 * **bin**
 * **expire**
* <code>drupal_render_cache_by_query()</code> provides a shortcut when the renderable array depends only on the results of a query.

</div>

<div class="slide reachable" id="render-cache2" markdown="1" style="left: 500px; top: 500px">

# Example

<?php
function forum_block_view($delta = '') {
  // [...] Generate $query.
  $block['subject'] = $title;
  // Cache based on the altered query. Enables us to cache with node access enabled.
  $block['content'] = drupal_render_cache_by_query($query, 'forum_block_view');
  $block['content']['#access'] = user_access('access content');
  return $block;
}
?>

<?php
/**
 * A #pre_render callback. Lists nodes based on the element's #query property.
 */
function forum_block_view_pre_render($elements) {
  $result = $elements['#query']->execute();
  if ($node_title_list = node_title_list($result)) {
    $elements['forum_list'] = $node_title_list;
    $elements['forum_more'] = array('#theme' => 'more_link', '#url' => 'forum', '#title' => t('Read the latest forum topics.'));
  }
  return $elements;
}
?>

</div>

<div class="slide reachable" id="render-cache-esi" markdown="1" style="left: 1000px;">

# Tying in ESI

* The renderable array code allows cache implementations to modify the rendered data inside <code>cache_set()</code>.
* It is possible to return an ESI command that will be dynamically replaced by the reverse proxy.

</div>

<div class="slide reachable" id="render-cache-nginx" markdown="1" style="left: 1000px; top: 300px">

# An example with Nginx

* The client requests a page.

* The request results in a page miss in Nginx cache.

* The cache implementation stores the rendered HTML in Memcache, and returns an ESI include command

* A partial HTML page with the ESI command is returned to Nginx

* Nginx executes the ESI command and get the rendered HTML directly from the Memcache and embeds it into the page.

</div>


</div>

